#========================================================================================
# package      DE_EPAGES::Catalog::t::RestApiIntegrationTest
# base         DE_EPAGES::Shop::API::TestImportXML
# state        public
#----------------------------------------------------------------------------------------
# description  Integration test for RestApi resources.
#               Checks if requests are matched by the correct resources.
#========================================================================================
package DE_EPAGES::Catalog::t::RestApiIntegrationTest;
use base DE_EPAGES::Shop::API::TestImportXML;

use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::WebService::API::ResourceRegistry;
use HTTP::Request;
use Test::Exception;
use Test::More;

use strict;



sub shopAlias {
    return 'TestShop';
}



sub importXMLPath {
    return 'import.xml';
}



sub test {
    my $self = shift;

    $self->test_get_category();
    $self->test_get_category_by_id();
    $self->test_put_category_by_id();
    $self->test_delete_category_by_id();
    $self->test_post_subcategory();

    $self->test_post_productCategoryAssignment();
    $self->test_delete_productCategoryAssignment();

    $self->test_get_categorySequence();
    $self->test_put_categorySequence();

    done_testing();
}



sub test_get_category {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('GET', '/api/categories/?locale=de_DE', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Catalog::API::Resource::Category', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Content::API::Object::StartPage', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if locale was extracted correctly");
}



sub test_get_category_by_id {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $CategoryPath = $self->shopPath().'/Categories/Shirts';
    my $Category = LoadObjectByPath($CategoryPath);
    my $Request = $self->_createRequest('GET', '/api/categories/'.$Category->guid().'/?locale=de_DE', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Catalog::API::Resource::Category', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Content::API::Object::Category', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if locale was extracted correctly");
}



sub test_put_category_by_id {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $CategoryPath = $self->shopPath().'/Categories/Shirts';
    my $Category = LoadObjectByPath($CategoryPath);
    my $Request = $self->_createRequest('PUT', '/api/categories/'.$Category->guid().'/?locale=de_DE', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Catalog::API::Resource::Category', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'put', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Content::API::Object::Category', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if locale was extracted correctly");
}



sub test_delete_category_by_id {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $CategoryPath = $self->shopPath().'/Categories/Shirts';
    my $Category = LoadObjectByPath($CategoryPath);
    my $Request = $self->_createRequest('DELETE', '/api/categories/'.$Category->guid(), 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Catalog::API::Resource::Category', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'delete', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Content::API::Object::Category', "$SubTestName - test if object was extracted correctly");
}



sub test_post_subcategory {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $CategoryPath = $self->shopPath().'/Categories/Shirts';
    my $Category = LoadObjectByPath($CategoryPath);
    my $Request = $self->_createRequest('POST', '/api/categories/'.$Category->guid().'/?locale=de_DE', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Catalog::API::Resource::Category', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'post', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Content::API::Object::Category', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if locale was extracted correctly");
}



sub test_post_productCategoryAssignment {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('POST', '/api/products-categories/', 'application/x-www-form-urlencoded');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Catalog::API::Resource::CategoriesProductsMapResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'post', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
}



sub test_delete_productCategoryAssignment {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('DELETE', '/api/products-categories/', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Catalog::API::Resource::CategoriesProductsMapResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'delete', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
}



sub test_get_categorySequence {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $CategoryPath = $self->shopPath().'/Categories/Shirts';
    my $Category = LoadObjectByPath($CategoryPath);
    my $Request = $self->_createRequest('GET', '/api/categories/'.$Category->guid().'/sequence', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Catalog::API::Resource::CategorySequence', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Content::API::Object::Category', "$SubTestName - test if object was extracted correctly");
}



sub test_put_categorySequence {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $CategoryPath = $self->shopPath().'/Categories/Shirts';
    my $Category = LoadObjectByPath($CategoryPath);
    my $Request = $self->_createRequest('PUT', '/api/categories/'.$Category->guid().'/sequence', 'application/vnd.epages.v1+json', '[]');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Catalog::API::Resource::CategorySequence', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'put', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Content::API::Object::Category', "$SubTestName - test if object was extracted correctly");
}



sub _createRequest {
    my $self = shift;
    my ($Method, $Route, $Accept, $Content) = @_;

    my $URL = 'http://www.example.com'.$Route;
    my $Request = HTTP::Request->new($Method => $URL);

    if(defined $Accept) {
        $Request->header( 'Accept' => $Accept );
    }
    if(defined $Content) {
        warn "set content: $Content \n";
        $Request->content( $Content );
    }

    return $Request;
}




__PACKAGE__->runScript();

1;