#======================================================================================================================
# package      DE_EPAGES::MultiStore::t::Synchronization
# base         Exporter
# state        public
#----------------------------------------------------------------------------------------------------------------------
# description  Synchronization integration tests
#======================================================================================================================
package DE_EPAGES::MultiStore::t::Synchronization;
use base Exporter;

use strict;

use Test::More;
use Getopt::Long;

use DE_EPAGES::Core::API::Script qw (
    RunScript
);
use DE_EPAGES::Database::API::Connection qw (
    GetCurrentDBHandle
);
use DE_EPAGES::Object::API::Factory qw (
    LoadRootObject
    LoadObjectByPath
    ExistsObjectByPath
);
use DE_EPAGES::WebService::API::Client;
use DE_EPAGES::Core::API::Error qw (
    ExistsError
    GetError
);
use DE_EPAGES::Core::API::Log qw (
    LogError
);

## no critic qw(DE_EPAGES::Style::RequireCorrectImports)
use DE_EPAGES::MultiStoreProvider::API::ShopRef qw (
    AssignSubShopToMasterShop
    UnassignSubShopFromMasterShop
    UpdateRemoteSubShop
);
use DE_EPAGES::MultiStore::API::Test::HelperFunctions qw (
    RunTestOnStore
    SetTestLanguageID
    SetVerbose
    GetVerbose
    Output
);
use DE_EPAGES::MultiStore::API::Test::SyncFunctions qw (
    ExportTestMasterShop
);
use DE_EPAGES::MultiStore::API::Test::TestSuite_Categories qw (
    RunTestSuite_CategorySynchronization
);

use DE_EPAGES::MultiStore::API::Test::TestSuite_Products qw (
    RunTestSuite_ProductSynchronization
);

use DE_EPAGES::MultiStore::API::Test::TestSuite_ProductTypes qw (
    RunTestSuite_ProductTypeSynchronization
);

my $hTestContext = {
    'Site' => {
        'Storename'         => 'Site',
        'Provider'          => 'Distributor',
        'ProviderLogin'     => '/Providers/Distributor/Users/admin',
        'ProviderPassword'  => 'admin',
        'ProxyURN'          => 'urn://epages.de/WebService/ShopConfigService/2009/09',
        'ProxyURL'          => undef,
        'ProxyLogin'        => undef,
        'ProxyPassword'     => undef,
    },
    'MasterShop' => {
        'Storename'         => 'Master',
        'ProxyURL'          => undef,
        'ProxyLogin'        => undef,
        'ProxyPassword'     => undef,
        'ShopType'          => 'Demo',
        'ShopAlias'         => 'Test-MasterShop-IPc90V1RhEqsuAepy130nA',
    },
    'SubShop'=> {
        'Storename'         => 'MultiStore',
        'ProxyURL'          => undef,
        'ProxyLogin'        => undef,
        'ProxyPassword'     => undef,
        'ShopType'          => 'ERetail1000',
        'ShopAlias'         => 'Test-SubShop-HCtD2YSWxkybyCe6Ssvz4A',
    },
    'Verbose'       => GetVerbose(),
    'NoCleanUp'     => 0,
    'OnlyCleanUp'   => 0
};

#======================================================================================================================
# Main
#======================================================================================================================
sub Main {

    ParseCmdLineParameters();
    SetVerbose($hTestContext->{'Verbose'});
    InitConnection();
    if (IsOnlyCleanUpRequested()) {
        CleanUpTest();
    } else {
        RunTest();
    }

    return;
}

#======================================================================================================================
# parseCmdLineParameters
#======================================================================================================================
sub ParseCmdLineParameters {

    my $Help;

    GetOptions(
        'help'          => \$Help,
        'master=s'      => \$hTestContext->{'Master'}->{'Storename'},
        'subshop=s'     => \$hTestContext->{'Subshop'}->{'Storename'},
        'verbose=s'     => \$hTestContext->{'Verbose'},
        'noclean'       => \$hTestContext->{'NoCleanUp'},
        'cleanup'       => \$hTestContext->{'OnlyCleanUp'},
    );

    usage() if $Help;

    return;
}

#======================================================================================================================
# InitConnection
#======================================================================================================================
sub InitConnection {

    GetProxiesUrlInfo();
    GetShopConfigService();

    return;
}

#======================================================================================================================
# GetProxiesUrlInfo
#======================================================================================================================
sub GetProxiesUrlInfo {

    RunTestOnStore(
        $hTestContext->{'Site'}->{'Storename'},
        sub {
            GetProxyUrlInfo($hTestContext->{'Site'});
            GetProxyUrlInfo($hTestContext->{'MasterShop'});
            GetProxyUrlInfo($hTestContext->{'SubShop'});
        }
    );

    return;
}

#======================================================================================================================
# GetProxyUrlInfo
#======================================================================================================================
sub GetProxyUrlInfo {
    my ($hInfo) = @_;

    my $Store = LoadRootObject()->folder('Stores')->child($hInfo->{'Storename'});
    my $hResult = $Store->get([ 'WebServiceProxy', 'WebServiceLogin', 'WebServicePassword' ]);
    $hInfo->{'ProxyURL'} = $hResult->{'WebServiceProxy'};
    $hInfo->{'ProxyLogin'} = $hResult->{'WebServiceLogin'};
    $hInfo->{'ProxyPassword'} = $hResult->{'WebServicePassword'};

    return;
}

#======================================================================================================================
# GetShopConfigService
#======================================================================================================================
sub GetShopConfigService {

    my $hSiteInfo = $hTestContext->{'Site'};
    my $ShopConfigService = DE_EPAGES::WebService::API::Client->uri($hSiteInfo->{'ProxyURN'});
    $ShopConfigService->proxy($hSiteInfo->{'ProxyURL'}, timeout => 900);
    $ShopConfigService->userinfo($hSiteInfo->{'ProviderLogin'}.':'.$hSiteInfo->{'ProviderPassword'});
    $hSiteInfo->{'ShopConfigService'} = $ShopConfigService;

    return;
}

#======================================================================================================================
# RunTest
#======================================================================================================================
sub RunTest {

    eval {
        InitTest();
        RunAllTestSuite();
        done_testing();
    };
    if (ExistsError()) {
        my $Error = GetError();
        print STDERR 'ERROR: '.$Error->shortMessage."\n";
        LogError($Error);
    }

    if(IsCleanUpEnabled()) {
        CleanUpTest();
    }

    return;
}

#======================================================================================================================
# InitTest
#======================================================================================================================
sub InitTest {
    Output('Initialize Test');

    CleanUpTest();
    UnbindTestShops();
    DeleteTestSubShop();
    CreateTestMasterShop();
    CreateTestSubShop();
    SetupTestMasterShop();
    SetupTestSubShop();
    BindTestShops();
    SetTestLanguageID($hTestContext->{'Site'}->{'Storename'});
    ExportTestMasterShop($hTestContext, 1);

    return;
}

#======================================================================================================================
# CleanUpTest
#======================================================================================================================
sub CleanUpTest {
    Output('Clean Up Test');

    UnbindTestShops();
    DeleteTestSubShop();
    DeleteTestMasterShop();

    return;
}

#======================================================================================================================
# CreateTestMasterShop
#======================================================================================================================
sub CreateTestMasterShop {

    CreateTestShop($hTestContext->{'MasterShop'});

    return;
}

#======================================================================================================================
# CreateTestSubShop
#======================================================================================================================
sub CreateTestSubShop() {

    CreateTestShop($hTestContext->{'SubShop'});

    return;
}

#======================================================================================================================
# CreateTestShop
#======================================================================================================================
sub CreateTestShop {
    my ($hShopInfo) = @_;

    my $ShopAlias = $hShopInfo->{'ShopAlias'};
    Output("Create test Shop '$ShopAlias'");

    my $ShopConfigService = $hTestContext->{'Site'}->{'ShopConfigService'};
    $ShopConfigService->create({
        'Database'              => $hShopInfo->{'Storename'},
        'ShopType'              => $hShopInfo->{'ShopType'},
        'ShopAlias'             => $ShopAlias,
        'Alias'                 => $ShopAlias,
        'IsInternalTestShop'    => 1,
    });

    return;
}

#======================================================================================================================
# SetupTestMasterShop
#======================================================================================================================
sub SetupTestMasterShop {

    Output('Setup MasterShop');

    my $hMasterInfo = $hTestContext->{'MasterShop'};
    RunTestOnStore(
        $hMasterInfo->{'Storename'},
        sub {
            my $hSiteInfo = $hTestContext->{'Site'};
            my $SiteName = $hSiteInfo->{'Storename'};
            my $ShopAlias = $hMasterInfo->{'ShopAlias'};
            my $ShopPath = "/Shops/$ShopAlias";
            my $Shop = LoadObjectByPath($ShopPath);
            $Shop->featureMaxValue('MultiStoreMaster', 1);
            $Shop->set({
                'MS_WebServiceURL' => {
                    $SiteName   => $hSiteInfo->{'ProxyURL'},
                    $ShopAlias  => $hMasterInfo->{'ProxyURL'},
                },
                'MS_WebServiceUser' => {
                    $SiteName   => $hSiteInfo->{'ProxyLogin'},
                    $ShopAlias  => $hMasterInfo->{'ProxyLogin'},
                },
                'MS_WebServicePassword' => {
                    $SiteName   => $hSiteInfo->{'ProxyPassword'},
                    $ShopAlias  => $hMasterInfo->{'ProxyPassword'},
                },
            });
            CleanObjectUpdateTableByShopID($Shop->id);
        }
    );

    return;
}

#======================================================================================================================
# SetupTestMasterShop
#======================================================================================================================
sub CleanObjectUpdateTableByShopID {
    my ($ShopID) = @_;

    my $dbi = GetCurrentDBHandle();
    $dbi->do('DELETE FROM objectupdate WHERE shopid = ?', [ $ShopID ]);

    return;
}

#======================================================================================================================
# SetupTestSubShop
#======================================================================================================================
sub SetupTestSubShop {

    Output('Setup SubShop');

    my $hSubShopInfo = $hTestContext->{'SubShop'};
    RunTestOnStore(
        $hSubShopInfo->{'Storename'},
        sub {
            my $hSiteInfo = $hTestContext->{'Site'};
            my $SiteName = $hSiteInfo->{'Storename'};
            my $ShopAlias = $hSubShopInfo->{'ShopAlias'};
            my $Shop = LoadObjectByPath('/Shops')->child($ShopAlias);
            $Shop->set({
                'MS_WebServiceURL' => {
                    $SiteName   => $hSiteInfo->{'ProxyURL'},
                },
                'MS_WebServiceUser' => {
                    $SiteName   => $hSiteInfo->{'ProxyLogin'},
                },
                'MS_WebServicePassword' => {
                    $SiteName   => $hSiteInfo->{'ProxyPassword'},
                },
            });
        }
    );

    return;
}

#======================================================================================================================
# DeleteTestSubShop
#======================================================================================================================
sub DeleteTestSubShop() {

    DeleteTestShop($hTestContext->{'SubShop'});

    return;
}

#======================================================================================================================
# DeleteTestMasterShop
#======================================================================================================================
sub DeleteTestMasterShop() {

    DeleteTestShop($hTestContext->{'MasterShop'});

    return;
}

#======================================================================================================================
# DeleteTestShop
#======================================================================================================================
sub DeleteTestShop() {
    my ($hShopInfo) = @_;

    my $ShopAlias = $hShopInfo->{'ShopAlias'};
    Output("Delete test Shop '$ShopAlias'");

    my $hSiteInfo = $hTestContext->{'Site'};
    RunTestOnStore(
        $hSiteInfo->{'Storename'},
        sub {
            my $Provider = $hSiteInfo->{'Provider'};
            my $ShopRefPath = "/Providers/$Provider/ShopRefs/$ShopAlias";
            if (ExistsObjectByPath($ShopRefPath)) {
                LoadObjectByPath($ShopRefPath)->delete();
            }
        }
    );

    return;
}

#======================================================================================================================
# UnbindTestShops
#======================================================================================================================
sub UnbindTestShops {

    Output('Unbind Test Shops');

    my $hSiteInfo = $hTestContext->{'Site'};
    RunTestOnStore(
        $hSiteInfo->{'Storename'},
        sub {
            my $Provider = $hSiteInfo->{'Provider'};
            my $SubShop = $hTestContext->{'SubShop'}->{'ShopAlias'};
            my $SubShopRefPath = "/Providers/$Provider/ShopRefs/$SubShop";
            if (ExistsObjectByPath($SubShopRefPath)) {
                UnassignSubShopFromMasterShop(LoadObjectByPath($SubShopRefPath));
            }
        }
    );

    return;
}

#======================================================================================================================
# BindTestShops
#======================================================================================================================
sub BindTestShops() {

    Output('Bind Test Shops');

    my $hSiteInfo = $hTestContext->{'Site'};
    RunTestOnStore(
        $hSiteInfo->{'Storename'},
        sub {
            my $Provider = $hSiteInfo->{'Provider'};
            my $MasterShopPath = "/Providers/$Provider/ShopRefs/".$hTestContext->{'MasterShop'}->{'ShopAlias'};
            my $SubShopPath = "/Providers/$Provider/ShopRefs/".$hTestContext->{'SubShop'}->{'ShopAlias'};
            my $MasterShopRef = LoadObjectByPath($MasterShopPath);
            my $SubShopRef = LoadObjectByPath($SubShopPath);
            AssignSubShopToMasterShop($MasterShopRef, $SubShopRef);
            AssignMasterShopToSubShop($SubShopRef, $MasterShopRef);
        }
    );

    return;
}

#======================================================================================================================
# AssignMasterShopToSubShop
#======================================================================================================================
sub AssignMasterShopToSubShop {
    my ($SubShopRef, $MasterShopRef) = @_;

    my $hSubShopInfo = $hTestContext->{'SubShop'};
    my $hParams = {
        'MasterShop'            => $MasterShopRef->alias,
        'MS_WebServiceLogging'  => 1,
        'MS_WebServiceURL'      => $hSubShopInfo->{'ProxyURL'},
        'MS_WebServiceUser'     => $hSubShopInfo->{'ProxyLogin'},
        'MS_WebServicePassword' => $hSubShopInfo->{'ProxyPassword'},
        'OrderIntegrationType'  => 'OrderSplitting',
    };

    UpdateRemoteSubShop($SubShopRef, $hParams);

    return;
}

#======================================================================================================================
# IsOnlyCleanUpRequested
#======================================================================================================================
sub IsOnlyCleanUpRequested() {

    return $hTestContext->{'OnlyCleanUp'};
}

#======================================================================================================================
# IsCleanUpEnabled
#======================================================================================================================
sub IsCleanUpEnabled() {

    return not $hTestContext->{'NoCleanUp'};
}

#======================================================================================================================
# RunAllTestSuite
#======================================================================================================================
sub RunAllTestSuite {

    RunTestSuite_CategorySynchronization($hTestContext);
    RunTestSuite_ProductSynchronization($hTestContext);
    RunTestSuite_ProductTypeSynchronization($hTestContext);

    return;
}

#======================================================================================================================
# usage
#======================================================================================================================
sub usage {
    my ($ErrorMessage) = @_;
    print STDERR $ErrorMessage . "\n" if ($ErrorMessage);

    print <<END_USAGE;

Purpose:
    Tests the Master <-> SubShop synchronization

Usage:
    perl $0 [options] [flags]

options:
    -master            Master Storename                           (default: Master)
    -subshop           Subshops Storename                         (default: MultiStore)
    -verbose           Level for extra info about test execution  (default: none)
                          Available values are [ none, info, all ] where:
                            none    No output at all
                            info    Only basic messages about test
                            all     Everything, even synchronization output

flags:
    -help              Shows the command line options
    -noclean           Keep test objects (skip cleanup)
    -cleanup           Only clean existing test shops

Example:
    perl $0 -verbose -master StoreMasterShops -subshop StoreSubShops

END_USAGE
    exit 2;
}

#======================================================================================================================
RunScript( 'Sub' => \&Main );
