#========================================================================================
# package      DE_EPAGES::Order::t::unit::BasketPatch
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Order::API::BasketPatch
#========================================================================================
package DE_EPAGES::Order::t::unit::BasketPatch;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Order::API::BasketPatch;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw (GetParametersFromMockifyCall WasCalled);
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_addProductLineItem();
    $self->test_modifyProductLineItem();
    $self->test_removeProductLineItem();

    $self->test_addBillingAddress();
    $self->test_addShippingAddress();

    $self->test_removeBillingAddress();
    $self->test_removeShippingAddress();

    done_testing();
}



sub test_addProductLineItem {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket($self->_createLineItemContainer());
    my $BasketPatch = $self->_createBasketPatch_addProductLineItem($Basket);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/lineItemContainer/productLineItems',
        'value' => {
            'productId' => 'myProduct123',
            'quantity' => '1',
        },
    };

    $BasketPatch->patch($hPatchOperation);

    my $aProdudctAddToContainerCallParams = GetParametersFromMockifyCall($self->{'__objectsToCheck'}->{'Product'}, 'addToContainer');
    my $hExpected2ndAddToContainerParam = {
        'Quantity' => 1
    };
    is($aProdudctAddToContainerCallParams->[0]->guid(), 'line-item-container-guid', "$SubTestName - test if 1st addToContainer is correct line item container");
    is_deeply($aProdudctAddToContainerCallParams->[1], $hExpected2ndAddToContainerParam, "$SubTestName - test if 2nd addToContainer param is as expected.");

    my $LineItemContainer = $self->{'__objectsToCheck'}->{'LineItemContainer'};
    ok(WasCalled($LineItemContainer, 'recalculate'), "$SubTestName - test if recalculate on LineItemContainer was called.");
}



sub test_modifyProductLineItem {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket($self->_createLineItemContainer());
    my $BasketPatch = $self->_createBasketPatch_modifyProductLineItem($Basket);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/lineItemContainer/productLineItems/Line-Item-Product-GUID',
        'value' => {
            'quantity' => '11',
        }
    };

    $BasketPatch->patch($hPatchOperation);

    my $LineItemProduct = $self->{'__objectsToCheck'}->{'LineItemProduct_modifyProductLineItem'};
    is($LineItemProduct->get('Quantity'), 11, "$SubTestName - test if quantity was updated."); ## no critic (ProhibitMagicNumbers)

    my $LineItemContainer = $self->{'__objectsToCheck'}->{'LineItemContainer'};
    ok(WasCalled($LineItemContainer, 'recalculate'), "$SubTestName - test if recalculate on LineItemContainer was called.");
}



sub test_removeProductLineItem {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket($self->_createLineItemContainer());
    my $BasketPatch = $self->_createBasketPatch_removeProductLineItem($Basket);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/lineItemContainer/productLineItems/Line-Item-Product-GUID',
    };

    $BasketPatch->patch($hPatchOperation);

    my $LineItemProduct = $self->{'__objectsToCheck'}->{'LineItemProduct_removeProductLineItem'};
    ok(WasCalled($LineItemProduct, 'delete'), "$SubTestName - test if delete on LineItemProduct was called.");
    my $LineItemContainer = $self->{'__objectsToCheck'}->{'LineItemContainer'};
    ok(WasCalled($LineItemContainer, 'recalculate'), "$SubTestName - test if recalculate on LineItemContainer was called.");
}



sub test_addBillingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket();
    my $BasketPatch = $self->_createBasketPatch_AddressPatch($Basket);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/billingAddress',
        'value' => {
            'firstName' => 'Joe',
            'lastName' => 'Doe',
        },
    };

    $BasketPatch->patch($hPatchOperation);

    my $BasketAddress = $self->{'__objectsToCheck'}->{'BasketAddress'};
    my $setAddressCallParams = GetParametersFromMockifyCall($BasketAddress, 'setAddress');
    my $hExpectedAddressData = {
        'firstName' => 'Joe',
        'lastName' => 'Doe',
    };
    is_deeply($setAddressCallParams->[0], $hExpectedAddressData, "$SubTestName - test if correct address data was stored.");
    is($setAddressCallParams->[1], 'Billing', "$SubTestName - test address type was billing");
}



sub test_removeBillingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket();
    my $BasketPatch = $self->_createBasketPatch_AddressPatch($Basket);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/billingAddress',
    };

    $BasketPatch->patch($hPatchOperation);

    my $BasketAddress = $self->{'__objectsToCheck'}->{'BasketAddress'};
    my $setAddressCallParams = GetParametersFromMockifyCall($BasketAddress, 'removeAddress');
    is($setAddressCallParams->[0], 'Billing', "$SubTestName - test address type was billing");
}



sub test_addShippingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket();
    my $BasketPatch = $self->_createBasketPatch_AddressPatch($Basket);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/shippingAddress',
        'value' => {
            'firstName' => 'Joe',
            'lastName' => 'Doe',
        },
    };

    $BasketPatch->patch($hPatchOperation);

    my $BasketAddress = $self->{'__objectsToCheck'}->{'BasketAddress'};
    my $setAddressCallParams = GetParametersFromMockifyCall($BasketAddress, 'setAddress');
    my $hExpectedAddressData = {
        'firstName' => 'Joe',
        'lastName' => 'Doe',
    };
    is_deeply($setAddressCallParams->[0], $hExpectedAddressData, "$SubTestName - test if correct address data was stored.");
    is($setAddressCallParams->[1], 'Shipping', "$SubTestName - test address type was shipping");
}



sub test_removeShippingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket();
    my $BasketPatch = $self->_createBasketPatch_AddressPatch($Basket);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/shippingAddress',
    };

    $BasketPatch->patch($hPatchOperation);

    my $BasketAddress = $self->{'__objectsToCheck'}->{'BasketAddress'};
    my $setAddressCallParams = GetParametersFromMockifyCall($BasketAddress, 'removeAddress');
    is($setAddressCallParams->[0], 'Shipping', "$SubTestName - test address type was shipping");
}



### helper methods ###



sub _createBasketPatch_AddressPatch {
    my $self = shift;
    my ($Basket) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Order::API::BasketPatch',
        [$Basket]
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_createBasketAddress',
        $self->_createBasketAddress(),
        [{'object' => 'DE_EPAGES::Order::API::Object::Basket'}]
    );
    return $Mockify->getMockObject();
}



sub _createBasketPatch_addProductLineItem {
    my $self = shift;
    my ($Basket) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::Order::API::BasketPatch', [$Basket] );
    $Mockify->addMockWithReturnValueAndParameterCheck( '_loadShopObjectByGUID', $self->_createProduct(), ['object', {'string'=>'myProduct123'}, 'string'] );
    $Mockify->addMockWithReturnValueAndParameterCheck( '_ValidateStockLevelSettings', undef, [{'object' => 'DE_EPAGES::Product::API::Object::Product'}, {'string' => '1'}] );
    return $Mockify->getMockObject();
}



sub _createBasketPatch_modifyProductLineItem {
    my $self = shift;
    my ($Basket) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::Order::API::BasketPatch', [$Basket] );
    $Mockify->addMockWithReturnValueAndParameterCheck( '_loadShopObjectByGUID', $self->_createLineItemProduct_modifyProductLineItem(), ['object', {'string'=>'Line-Item-Product-GUID'}, 'string'] );
    $Mockify->addMockWithReturnValueAndParameterCheck( '_ValidateStockLevelSettings', undef, [{'object' => 'DE_EPAGES::Product::API::Object::Product'}, {'string' => '11'}] );
    return $Mockify->getMockObject();
}



sub _createBasketPatch_removeProductLineItem {
    my $self = shift;
    my ($Basket) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Order::API::BasketPatch',
        [$Basket]
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_loadShopObjectByGUID',
        $self->_createLineItemProduct_removeProductLineItem(),
        ['object', {'string'=>'Line-Item-Product-GUID'}, 'string']
    );
    return $Mockify->getMockObject();
}



sub _createBasket {
    my $self = shift;
    my ($LineItemContainer) = @_;
    my $hAtributes = {
        'GUID' => 'basket-guid-123',
        'Site' => $self->_createShop(),
        'LineItemContainer' => $LineItemContainer,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket', $hAtributes);
}



sub _createLineItemContainer {
    my $self = shift;
    my $hAttributes =  {
        'GUID' => 'line-item-container-guid',
        'Parent' => $self->_createBasket(),
        'LineItems' => undef,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer',
        $hAttributes
    );
    $Mockify->addMockWithReturnValue('recalculate', undef);
    my $LineItemContainer = $Mockify->getMockObject();
    $self->{'__objectsToCheck'}->{'LineItemContainer'} = $LineItemContainer;
    return $LineItemContainer;
}



sub _createShop {
    my $self = shift;
    my $hAtributes = {
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAtributes);
}



sub _createProduct {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::Product::API::Object::Product', [] );
    $Mockify->addMockWithReturnValueAndParameterCheck('addToContainer', undef, ['object', 'hashref']);
    my $Product = $Mockify->getMockObject();
    $self->{'__objectsToCheck'}->{'Product'} = $Product;
    return $Product;
}



sub _createLineItemProduct_modifyProductLineItem {
    my $self = shift;
    my $hAtributes = {
        'GUID' => 'line-item-guid',
        'Parent' => $self->{'__objectsToCheck'}->{'LineItemContainer'},
        'Product' => $self->_createProduct(),
    };
    my $LineItemProduct = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Product::API::Object::LineItemProduct', $hAtributes);
    $self->{'__objectsToCheck'}->{'LineItemProduct_modifyProductLineItem'} = $LineItemProduct;
    return $LineItemProduct;
}



sub _createLineItemProduct_removeProductLineItem {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Product::API::Object::LineItemProduct',
        []
    );
    $Mockify->addMockWithReturnValue('delete', undef);
    $Mockify->addMockWithReturnValue('guid', 'line-item-guid');
    $Mockify->addMockWithReturnValue('parent', $self->{'__objectsToCheck'}->{'LineItemContainer'});
    my $LineItemProduct = $Mockify->getMockObject();
    $self->{'__objectsToCheck'}->{'LineItemProduct_removeProductLineItem'} = $LineItemProduct;
    return $LineItemProduct;
}



sub _createBasketAddress {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Order::API::BasketAddress',
        []
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        'setAddress',
        undef,
        ['hashref', 'string']
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        'removeAddress',
        undef,
        ['string']
    );
    my $BasketAddress = $Mockify->getMockObject();
    $self->{'__objectsToCheck'}->{'BasketAddress'} = $BasketAddress;
    return $BasketAddress;
}

__PACKAGE__->runScript();