#========================================================================================
# package      DE_EPAGES::Product::t::RestApiIntegrationTest
# base         DE_EPAGES::Shop::API::TestImportXML
# state        public
#----------------------------------------------------------------------------------------
# description  Integration test for RestApi resources.
#               Checks if requests are matched by the correct resources.
#========================================================================================
package DE_EPAGES::Product::t::RestApiIntegrationTest;
use base DE_EPAGES::Shop::API::TestImportXML;

use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::WebService::API::ResourceRegistry;
use HTTP::Request;
use Test::Exception;
use Test::More;

use strict;



sub shopAlias {
    return 'TestShop';
}



sub importXMLPath {
    return 'import.xml';
}



sub test {
    my $self = shift;

    $self->test_get_product_customattributes();

    $self->test_get_products();

    $self->test_get_product();
    $self->test_delete_product();
    $self->test_patch_product();

    $self->test_get_product_categories();

    $self->test_get_product_lowestprice();

    $self->test_post_product_slideshow();

    $self->test_delete_product_slideshow_image();

    $self->test_get_product_slideshow_sequence();
    $self->test_put_product_slideshow_sequence();

    $self->test_get_product_variations();

    $self->test_get_updated_products();

    $self->test_get_watched_products();

    done_testing();
}



sub test_get_product_customattributes {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $ProductPath = $self->shopPath().'/Products/MyJacket';
    my $Product = LoadObjectByPath($ProductPath);
    my $Request = $self->_createRequest('GET', '/api/products/'.$Product->guid().'/custom-attributes?locale=de_DE', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Product::API::Resource::CustomAttributes', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Product::API::Object::Product', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if locale was extracted correctly");
}



sub test_get_products {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('GET', '/api/products?locale=de_DE&currency=EUR&id=123&id=456', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Product::API::Resource::ProductCollection', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if locale was extracted correctly");
    is($aParams->[2], 'EUR', "$SubTestName - test if currency was extracted correctly");
    is_deeply($aParams->[3], [123, 456], "$SubTestName - test if GUIDs were extracted correctly");
}



sub test_get_product {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $ProductPath = $self->shopPath().'/Products/MyJacket';
    my $Product = LoadObjectByPath($ProductPath);
    my $Request = $self->_createRequest('GET', '/api/products/'.$Product->guid().'?locale=de_DE&currency=EUR', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Product::API::Resource::Product', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Product::API::Object::Product', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if locale was extracted correctly");
    is($aParams->[2], 'EUR', "$SubTestName - test if currency was extracted correctly");
}



sub test_delete_product {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $ProductPath = $self->shopPath().'/Products/MyJacket';
    my $Product = LoadObjectByPath($ProductPath);
    my $Request = $self->_createRequest('DELETE', '/api/products/'.$Product->guid());

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Product::API::Resource::Product', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'delete', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Product::API::Object::Product', "$SubTestName - test if object was extracted correctly");
}



sub test_patch_product {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $ProductPath = $self->shopPath().'/Products/MyJacket';
    my $Product = LoadObjectByPath($ProductPath);
    my $Request = $self->_createRequest('PATCH', '/api/products/'.$Product->guid().'?locale=de_DE&currency=EUR', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Product::API::Resource::Product', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'patch', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Product::API::Object::Product', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if locale was extracted correctly");
    is($aParams->[2], 'EUR', "$SubTestName - test if currency was extracted correctly");
}



sub test_get_product_categories {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $ProductPath = $self->shopPath().'/Products/MyJacket';
    my $Product = LoadObjectByPath($ProductPath);
    my $Request = $self->_createRequest('GET', '/api/products/'.$Product->guid().'/categories?locale=de_DE', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Product::API::Resource::ProductCategories', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Product::API::Object::Product', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if locale was extracted correctly");
}



sub test_get_product_lowestprice {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $ProductPath = $self->shopPath().'/Products/MyJacket';
    my $Product = LoadObjectByPath($ProductPath);
    my $Request = $self->_createRequest('GET', '/api/products/'.$Product->guid().'/lowest-price?locale=de_DE&currency=EUR', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Product::API::Resource::ProductLowestPrice', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Product::API::Object::Product', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if locale was extracted correctly");
    is($aParams->[2], 'EUR', "$SubTestName - test if currency was extracted correctly");
}



sub test_post_product_slideshow {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $ProductPath = $self->shopPath().'/Products/MyJacket';
    my $Product = LoadObjectByPath($ProductPath);
    my $Request = $self->_createRequest('POST', '/api/products/'.$Product->guid().'/slideshow?imagename=newimage.png', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Product::API::Resource::ProductSlideshow', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'post', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Product::API::Object::Product', "$SubTestName - test if object was extracted correctly");
    isa_ok($aParams->[1], 'HTTP::Request', "$SubTestName - test if request object was extracted correctly");
    is($aParams->[2], 'newimage.png', "$SubTestName - test if imagename was extracted correctly");
}



sub test_delete_product_slideshow_image {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $ProductPath = $self->shopPath().'/Products/MyJacket';
    my $Product = LoadObjectByPath($ProductPath);
    my $Request = $self->_createRequest('DELETE', '/api/products/'.$Product->guid().'/slideshow/newimage.png', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Product::API::Resource::ProductSlideshowImageDeletion', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'delete', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Product::API::Object::Product', "$SubTestName - test if object was extracted correctly");
    is_deeply($aParams->[1], ['newimage.png'], "$SubTestName - test if path params were extracted correctly");
}



sub test_get_product_slideshow_sequence {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $ProductPath = $self->shopPath().'/Products/MyJacket';
    my $Product = LoadObjectByPath($ProductPath);
    my $Request = $self->_createRequest('GET', '/api/products/'.$Product->guid().'/slideshow/sequence', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Product::API::Resource::ProductSlideshowSequence', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Product::API::Object::Product', "$SubTestName - test if object was extracted correctly");
}



sub test_put_product_slideshow_sequence {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $ProductPath = $self->shopPath().'/Products/MyJacket';
    my $Product = LoadObjectByPath($ProductPath);
    my $Request = $self->_createRequest('PUT', '/api/products/'.$Product->guid().'/slideshow/sequence', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Product::API::Resource::ProductSlideshowSequence', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'put', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Product::API::Object::Product', "$SubTestName - test if object was extracted correctly");
}



sub test_get_product_variations {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $ProductPath = $self->shopPath().'/Products/MyJacket';
    my $Product = LoadObjectByPath($ProductPath);
    my $Request = $self->_createRequest('GET', '/api/products/'.$Product->guid().'/variations?locale=de_DE&page=3&resultsPerPage=5', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Product::API::Resource::ProductVariation', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Product::API::Object::Product', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if locale was extracted correctly");
    is($aParams->[2], 3, "$SubTestName - test if page was extracted correctly");
    is($aParams->[3], 5, "$SubTestName - test if page size was extracted correctly");
}



sub test_get_updated_products {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $QueryParams = '?locale=de_DE&currency=EUR&page=3&resultsPerPage=5&attributesUpdatedSince=2016-02-16T16:30&stockLevelUpdatedSince=2016-03-14T15:00';
    my $Request = $self->_createRequest('GET', '/api/updated-products'.$QueryParams, 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Product::API::Resource::UpdatedProductsResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 5, "$SubTestName - test if page size was extracted correctly");
    is($aParams->[2], 3, "$SubTestName - test if page was extracted correctly");
    is($aParams->[3], 'de_DE', "$SubTestName - test if locale was extracted correctly");
    is($aParams->[4], 'EUR', "$SubTestName - test if currency was extracted correctly");
    is($aParams->[5], '2016-02-16T17:30:00', "$SubTestName - test if attributesUpdatedSince was extracted correctly");
    is($aParams->[6], '2016-03-14T16:00:00', "$SubTestName - test if stockLevelUpdatedSince was extracted correctly");
}



sub test_get_watched_products {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $QueryParams = '?locale=de_DE&currency=EUR&page=3&resultsPerPage=5';
    my $Request = $self->_createRequest('GET', '/api/watched-products'.$QueryParams, 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Product::API::Resource::WatchedProductsResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if locale was extracted correctly");
    is($aParams->[2], 5, "$SubTestName - test if page size was extracted correctly");
    is($aParams->[3], 3, "$SubTestName - test if page was extracted correctly");
}



sub _createRequest {
    my $self = shift;
    my ($Method, $Route, $Accept) = @_;

    my $URL = 'http://www.example.com'.$Route;
    my $Request = HTTP::Request->new($Method => $URL);

    if(defined $Accept) {
        $Request->header( 'Accept' => $Accept );
    }

    return $Request;
}



__PACKAGE__->runScript();

1;