#========================================================================================
# package      DE_EPAGES::SageOne::t::unit::RestService_Delete
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::SageOne::API::Webservice::RestService
#========================================================================================
package DE_EPAGES::SageOne::t::unit::RestService_Delete;
use base DE_EPAGES::Test::API::UnitTest::Test;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::EPagesJ::API::Object::RestClient;
use DE_EPAGES::SageOne::API::Webservice::RestService;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::EPagesJ::API::Object::HTTPClient;
use Test::More;
use Test::Exception;
use Test::MockObject;
use HTTP::Response;
use HTTP::Status qw (HTTP_OK);

use strict;

sub ErrorDefinition {
    return {
        'NativeDie' => 'malformed JSON string',
    };
}

sub test {
    my $self = shift;

    $self->_buildMocks();

    $self->test_DeleteRequestSuccessful();
    $self->test_DeleteRequestWebserviceError();
    $self->test_DeleteRequestUnknownError();

    $self->test_DecodeResponseJsonParserError();
    $self->test_DecodeResponseUnknownError();
    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_DeleteRequestSuccessful {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUTSuccessful');

    my $hResponse = $SUT->sendDeleteRequest('http://domain.com', {'accept' => 'application/JSON'});
    my $ExpectedJson = '{"result":"Body Ok"}';

    is_deeply($hResponse->content(), $ExpectedJson, "$SubTestName - Test if response returning correctly");
}

sub test_DeleteRequestWebserviceError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUTWebserviceError');

    throws_ok( sub {  $SUT->sendDeleteRequest('http://domain.com', {'accept' => 'application/JSON'}); },
                   qr/WebserviceError: StatusCode: \(undef\)/,
                   "$SubTestName - test if Request result in error"
         );
}

sub test_DeleteRequestUnknownError {

    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUTUnknownError');

    throws_ok( sub {  $SUT->sendDeleteRequest('http://domain.com', {'accept' => 'application/JSON'}); },
                   qr/Unknown/,
                   "$SubTestName - test if DeleteRequest throws unknown error"
         );
}

sub test_DecodeResponseJsonParserError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUTDecodeError');
    my $Response = $self->_createResponse(HTTP_OK, 'not json');

    throws_ok( sub {  $SUT->decodeResponse($Response) },
                   qr/JsonParserError: malformed JSON/,
                   "$SubTestName - test if decoded response result in error"
         );
}

sub test_DecodeResponseUnknownError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUTDecodeUnknownError');
    my $Response = $self->_createResponse(HTTP_OK, '{"result":"Body Ok"}');
    throws_ok( sub {  $SUT->decodeResponse($Response); },
                   qr/Unknown/,
                   "$SubTestName - test if Decode Response throws unknown error"
         );
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildMocks {
    my $self = shift;

    my $Shop = $self->mockFactory()->buildShop({
            'Alias' => 'TestShop',
        })->getMockObject();

    $self->_buildMocksDeleteRequest($Shop);
    $self->_buildMocksDecodeResponse($Shop);
}

sub _buildMocksDeleteRequest {
    my $self = shift;
    my ($Shop) = @_;

    my $SUTSuccessful = $self->_createSUT($Shop, $self->_buildRestClientSuccessful());
    my $SUTWebserviceError = $self->_createSUT($Shop, $self->_buildRestClientWebserviceError());
    my $SUTUnknownError = $self->_createSUT($Shop, $self->_buildRestClientUnknownError());

    $self->memory()->add('SUTSuccessful', $SUTSuccessful);
    $self->memory()->add('SUTWebserviceError', $SUTWebserviceError);
    $self->memory()->add('SUTUnknownError', $SUTUnknownError);
    return;
}

sub _buildMocksDecodeResponse {
    my $self = shift;
    my ($Shop) = @_;

    my $Response = $self->_createResponse(HTTP_OK, '{"result":"Body Ok"}');
    my $SUTDecodeError = $self->_createSUT($Shop, $self->_buildRestClientDecodeError(), $Response);
    my $SUTDecodeUnknownError = $self->_createSUT($Shop, $self->_buildRestClientUnknownError());

    $self->memory()->add('SUTDecodeError', $SUTDecodeError);
    $self->memory()->add('SUTDecodeUnknownError', $SUTDecodeUnknownError);
    return;
}
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($Shop, $RestClient) = @_;

    my $aParameterList = [$Shop];

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::RestService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValue('_getRestClient', $RestClient);
    $Mockify->addMockWithReturnValue('_getLogger', $self->_createLogger());
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _buildRestClientSuccessful {
    my $self = shift;
    my ($Shop) = @_;

    my $HttpClient = $self->_createHTTPClientSuccessful();
    return $self->_createRestClient($Shop, $HttpClient);
}

sub _buildRestClientWebserviceError {
    my $self = shift;
    my ($Shop) = @_;

    my $HttpClient = $self->_createHTTPClientError();
    return $self->_createRestClient($Shop, $HttpClient);
}

sub _buildRestClientUnknownError {
    my $self = shift;
    my ($Shop) = @_;

    my $HttpClient = $self->_createHTTPClientSuccessful();
    return $self->_createRestClientUnknownError($Shop, $HttpClient);
}

sub _buildRestClientDecodeError {
    my $self = shift;
    my ($Shop) = @_;

    my $HttpClient = $self->_createHTTPClientSuccessful();
    return $self->_createRestClientDecodeError($Shop, $HttpClient);
}

#----------------------------------------------------------------------------------------
sub _createRestClient {
    my $self = shift;
    my ($Shop, $HttpClient) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::EPagesJ::API::Object::RestClient',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValue('client', $HttpClient);
    return $Mockify->getMockObject();
}

sub _createRestClientDecodeError {
    my $self = shift;
    my ($Shop, $HttpClient) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::EPagesJ::API::Object::RestClient',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValue('client', $HttpClient);
    $Mockify->addMock('decodeResponse', sub { Error('NativeDie')});

    return $Mockify->getMockObject();
}

sub _createRestClientUnknownError {
    my $self = shift;
    my ($Shop, $HttpClient) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::EPagesJ::API::Object::RestClient',
                   $aParameterList
              );
    $Mockify->addMock('client', sub { Error('Unknown')}); # For $SUTUnknownError
    $Mockify->addMock('decodeResponse', sub { Error('Unknown')}); # For $SUTDecodeUnknownError
    return $Mockify->getMockObject();
}

sub _createHTTPClientSuccessful {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::EPagesJ::API::Object::HTTPClient',
                   $aParameterList
              );

    my $Response = $self->_createResponse(HTTP_OK, '{"result":"Body Ok"}');
    $Mockify->addMockWithReturnValueAndParameterCheck('delete', $Response,
    [
        {'string' => 'http://domain.com'},
        {'hashref' => {'accept' => 'application/JSON'}}
    ]);
    return $Mockify->getMockObject();
}

sub _createHTTPClientError {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::EPagesJ::API::Object::HTTPClient',
                   $aParameterList
              );
    $Mockify->addMock('delete', sub {Error('REQUEST_FAILED')});

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createResponse {
    my $self = shift;
    my ($Code, $Content) = @_;
    my $Response = HTTP::Response->new($Code, 'all is fine');
    $Response->header('content-type' => 'application/json');
    $Response->content($Content);
    return $Response;
}

sub _createLogger {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::SageOne::API::Logger');
    $Mockify->addMockWithReturnValue('_isLoggingActivated', 1);
    $Mockify->addMockWithReturnValueAndParameterCheck('_prettify', '', [{'any'}]);
    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();