#========================================================================================
# package      DE_EPAGES::Spreedly::t::unit::RestClient
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for RestClient
#========================================================================================
package DE_EPAGES::Spreedly::t::unit::RestClient;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Spreedly::API::RestClient;
use DE_EPAGES::Spreedly::API::Tools;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( WasCalled );
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;

use strict;

use constant PAYMENT_INFO_TOKEN => '3EkAnf1vawsllJm40UJfYTyh1RT';
use constant SHOP_GUID          => '56C6F899-EF73-56A9-9314-7F000101C063';
use constant GATEWAY_NAME       => 'Barretina';
use constant AuthorizationCode  => 'ac_8NPktx3GXN4UBeZGsTEBXPZsv5koO0h5';

sub test {
    my $self = shift;

    $self->_test_BuildPurchaseData();
    $self->_test_MakePurchase();
    $self->_test_buildGatewayRequestBody_WithoutAdditionalParameters();
    $self->_test_buildGatewayRequestBody_WithAdditionalParameters();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub _test_BuildPurchaseData {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedRestClient  = $self->_buildRestClient('put');
    my $hOrder            = $self->_buildOrderData();
    my $hExpectedPurchase = $self->_buildPurchaseData($hOrder);
    my $hPurchase         = $MockedRestClient->_buildPurchaseData( PAYMENT_INFO_TOKEN, $hOrder );    ## no critic (ProhibitPrivateSubroutineUse)
    is_deeply( $hPurchase, $hExpectedPurchase, "$SubTestName - Purchase data matches expected structure." );

    return;
}

sub _test_MakePurchase {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedRestClient = $self->_buildRestClient('put');
    $MockedRestClient->{'HttpClient'} = _mockHTTPClient( { 'post' => '' } );
    my $hOrder = $self->_buildOrderData();
    my $hTransaction = $MockedRestClient->makePurchase( PAYMENT_INFO_TOKEN, $hOrder );
    ok( WasCalled( $MockedRestClient, '_buildPurchaseData' ), "$SubTestName - _buildPurchaseData has been called." );
    ok( $hTransaction->{'successful'},               "$SubTestName - Purchase is successful." );
    ok( isValid( $hTransaction->{'transactionNo'} ), "$SubTestName - Transaction number has been correctly retrieved." );

    return;
}

sub _test_buildGatewayRequestBody_WithoutAdditionalParameters {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedRestClient = $self->_buildRestClient('put');
    my $hGotRequestBody = $MockedRestClient->_buildGatewayRequestBody(GATEWAY_NAME); ## no critic (ProhibitPrivateSubroutineUse)
    my $hExpectedRequestBody = $self->_buildRequestBody_WithoutAdditionalParameters();
    is_deeply($hGotRequestBody, $hExpectedRequestBody, "$SubTestName - Request body has been built correctly.");

    return;
}

sub _test_buildGatewayRequestBody_WithAdditionalParameters {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedRestClient = $self->_buildRestClient('put');
    my $hAdditionalBodyParams = {
        'authorization_code' => AuthorizationCode,
    };
    my $hGotRequestBody = $MockedRestClient->_buildGatewayRequestBody(GATEWAY_NAME, $hAdditionalBodyParams); ## no critic (ProhibitPrivateSubroutineUse)
    my $hExpectedRequestBody = $self->_buildRequestBody_WithAdditionalParameters();
    is_deeply($hGotRequestBody, $hExpectedRequestBody, "$SubTestName - Request body has been built correctly.");

    return;
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _buildOrderData {
    my $self = shift;

    my $hParams = {
        'amount'        => '100',
        'currency_code' => 'USD',
    };

    return $self->_createOrderData($hParams);
}

sub _buildPurchaseData {
    my $self = shift;
    my ($hOrder) = @_;

    my $Tools = DE_EPAGES::Spreedly::API::Tools->new();

    my $hParams = {
        'payment_method_token' => PAYMENT_INFO_TOKEN,
        'amount'               => $Tools->convertNumberToSpreedlyFormat( $hOrder->{'amount'} ),
        'currency_code'        => $hOrder->{'currency_code'},
    };

    return $self->_createPurchaseData($hParams);
}

sub _buildRequestBody_WithAdditionalParameters {
    my $self = shift;

    my $hRequest = $self->_buildRequestBody_WithoutAdditionalParameters();
    $hRequest->{'authorization_code'} = AuthorizationCode;

    return $hRequest;
}

sub _buildRequestBody_WithoutAdditionalParameters {
    my $self = shift;

    my $hRequest = {
        'shopGUID' => SHOP_GUID,
        'name'     => GATEWAY_NAME,
    };

    return $hRequest;
}

sub _buildRestClient {
    my $self = shift;
    my ($GwMethod) = @_;

    my $hParams = { 'Shop' => $self->_buildShop(), };

    return $self->_createRestClient($hParams, $GwMethod);
}

sub _buildShop {
    my $self = shift;

    my $hAttributes = { 'ShopTypeRef' => { 'Alias' => 'Demo' }, 'GUID' => SHOP_GUID };

    return $self->_createShop($hAttributes);
}

sub _createOrderData {
    my $self = shift;
    my ($hParams) = @_;

    my $hOrder = {
        'amount'        => '100',
        'currency_code' => 'USD',
    };

    return $hOrder;
}

sub _createPurchaseData {
    my $self = shift;
    my ($hParams) = @_;

    my $hPurchase = {
        'transaction' => {
            'payment_method_token' => $hParams->{'payment_method_token'},
            'amount'               => $hParams->{'amount'},
            'currency_code'        => $hParams->{'currency_code'},
        }
    };

    return $hPurchase;
}

sub _createRestClient {
    my $self = shift;
    my ($hParams, $GwMethod) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Spreedly::API::RestClient');
    $Mockify->addMockWithReturnValue( '_LoadRootObject',           $self->_createSystem() );
    ($GwMethod eq 'put') ?
            $Mockify->addMockWithReturnValue( '_createPutGatewayEndpointURL', 'http://myfakeurl.cat/rs/payments/spreedly/gateways' )
            : $Mockify->addMockWithReturnValue( '_createGetGatewayEndpointURL', 'http://myfakeurl.cat/rs/payments/spreedly/gateways' );
    $Mockify->addMockWithReturnValueAndParameterCheck( '_createPurchaseEndpointURL',
        'http://myfakeurl.cat/rs/payments/spreedly/purchase?shopId=666&gatewayName=test',
        ['string'] );
    $Mockify->addMethodSpyWithParameterCheck( '_buildPurchaseData', [ 'string', 'hashref' ] );

    $Mockify->addMock(
        '_decodeFromResponse',
        sub {
            shift;
            my ($Response) = @_;
            return $self->_createTransactionOK();
        }
    );

    my $MockedRestClient = $Mockify->getMockObject();
    $MockedRestClient->{Shop} = $hParams->{'Shop'};

    return $MockedRestClient;
}

sub _createShop {
    my $self = shift;
    my ($hAttributes) = @_;

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hAttributes );
}

sub _createSystem {
    my $self = shift;

    my $hAttributes = {
        'EPagesJBaseURL' => 'http://myfakeurl.cat/rs/',
        'ProviderID'     => 'epages',
    };
    my $System = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::Site', $hAttributes );

    return $System;
}

sub _createTransactionOK {
    my $self = shift;

    my $hTransaction = {
        'successful'    => '1',
        'transactionNo' => '123434',
    };

    return $hTransaction;
}

sub _mockHTTPClient {
    my ($hMockMethods) = @_;
    my $mockRemote = Test::MockObject->new();
    $mockRemote->set_isa('DE_EPAGES::EPagesJ::API::Object::HTTPClient');
    $mockRemote->fake_module( 'DE_EPAGES::EPagesJ::API::Object::HTTPClient', %$hMockMethods );
    while ( my ( $method, $handler ) = each %$hMockMethods ) {
        $mockRemote->mock( $method, $handler );
    }
    return $mockRemote;
}

__PACKAGE__->runScript();

1;
