#========================================================================================
# package      DE_EPAGES::Spreedly::t::unit::UI_BasketOrder
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for UI::BasketOrder
#========================================================================================
package DE_EPAGES::Spreedly::t::unit::UI_BasketOrder;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Spreedly::UI::BasketOrder;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall WasCalled );
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;

use strict;

use constant PAYMENT_INFO_TOKEN => 'EWdi0wRvmXUX5pOqPgvhTUtV8g3';

sub test {
    my $self = shift;

    $self->_test_MakePurchase_SuccessfulTransaction();
    $self->_test_MakePurchase_UnsuccessfulTransaction();
    $self->_test_MakePurchase_WithError();
    $self->_test_RetrievePaymentInfoToken();
    $self->_test_RetrievePaymentInfoToken_EmptyToken();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub _test_MakePurchase_SuccessfulTransaction {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIBasketOrder = $self->_buildUIBasketOrder_SuccessfulTransaction();
    my $MockedServlet       = $self->_buildServlet();
    $MockedUIBasketOrder->makePurchase($MockedServlet);
    ok( WasCalled( $MockedUIBasketOrder, '_GetPaymentInfoToken' ),   "$SubTestName - _GetPaymentInfoToken has been called." );
    ok( WasCalled( $MockedUIBasketOrder, '_GetOrder' ),              "$SubTestName - _GetOrder has been called." );
    ok( WasCalled( $MockedUIBasketOrder, '_SaveTransactionNumber' ), "$SubTestName - _SaveTransactionNumber has been called." );

    return;
}

sub _test_MakePurchase_UnsuccessfulTransaction {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIBasketOrder = $self->_buildUIBasketOrder_UnsuccessfulTransaction();
    my $MockedServlet       = $self->_buildServlet();
    $MockedUIBasketOrder->makePurchase($MockedServlet);
    ok( WasCalled( $MockedUIBasketOrder, '_GetPaymentInfoToken' ), "$SubTestName - _GetPaymentInfoToken has been called." );
    ok( WasCalled( $MockedUIBasketOrder, '_GetOrder' ),            "$SubTestName - _GetOrder has been called." );
    ok( WasCalled( $MockedUIBasketOrder, '_SetPaymentAsFailed' ), "$SubTestName - _SetPaymentAsFailed has been called." );

    return;
}

sub _test_MakePurchase_WithError {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIBasketOrder = $self->_buildUIBasketOrder_WithError();
    my $MockedServlet       = $self->_buildServlet();
    throws_ok( sub { $MockedUIBasketOrder->makePurchase($MockedServlet); },
        qr/ConnectionError/, "$SubTestName - Threw a SpreedlyMakePurchaseError exception" );
    ok( WasCalled( $MockedUIBasketOrder, '_GetPaymentInfoToken' ), "$SubTestName - _GetPaymentInfoToken has been called." );
    ok( WasCalled( $MockedUIBasketOrder, '_GetOrder' ),            "$SubTestName - _GetOrder has been called." );
    ok( !WasCalled( $MockedUIBasketOrder, '_SaveTransactionNumber' ), "$SubTestName - _SaveTransactionNumber has not been called." );

    return;
}

sub _test_RetrievePaymentInfoToken {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIBasketOrder = $self->_createUIBasketOrder();
    my $MockedServlet       = $self->_buildServlet();
    $MockedUIBasketOrder->RetrievePaymentInfoToken($MockedServlet);
    my $Servlet = @{ GetParametersFromMockifyCall( $MockedUIBasketOrder, 'RetrievePaymentInfoToken' ) }[0];
    my $GotPaymentInfoToken = $Servlet->params('payment_method_token');
    is( $GotPaymentInfoToken, PAYMENT_INFO_TOKEN, "$SubTestName - Obtained expected payment information token." );

    return;
}

sub _test_RetrievePaymentInfoToken_EmptyToken {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIBasketOrder = $self->_createUIBasketOrder();
    my $MockedServlet       = $self->_buildServlet_EmptyToken();
    throws_ok( sub { $MockedUIBasketOrder->RetrievePaymentInfoToken($MockedServlet); },
        qr/ConnectionError/, "$SubTestName - Threw a ConnectionError exception" );
    my $Servlet                  = @{ GetParametersFromMockifyCall( $MockedUIBasketOrder, 'RetrievePaymentInfoToken' ) }[0];
    my $GotPaymentInfoToken      = $Servlet->params('payment_method_token');
    my $ExpectedPaymentInfoToken = undef;
    is( $GotPaymentInfoToken, $ExpectedPaymentInfoToken, "$SubTestName - Payment information token is empty." );

    return;
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _buildServlet {
    my $self             = shift;
    my $PaymentInfoToken = PAYMENT_INFO_TOKEN;

    return $self->_createServlet($PaymentInfoToken);
}

sub _buildServlet_EmptyToken {
    my $self = shift;

    return $self->_createServlet();
}

sub _buildUIBasketOrder_SuccessfulTransaction {
    my $self = shift;

    my $hReturnValues = {
        '_makePurchase' => {
            'successful'    => '1',
            'transactionNo' => '123434',
        },
        '_LogPayment'          => undef,
        '_GetPaymentInfoToken' => PAYMENT_INFO_TOKEN,
        '_createCustomerOrder' => $self->_buildCustomerOrder(),
    };

    my $ThrowsError = 0;

    return $self->_createUIBasketOrder( $hReturnValues, $ThrowsError );
}

sub _buildUIBasketOrder_UnsuccessfulTransaction {
    my $self = shift;

    my $hReturnValues = {
        '_makePurchase' => {
            'successful'    => '0',
            'transactionNo' => '123434',
        },
        '_LogPayment'          => undef,
        '_GetPaymentInfoToken' => PAYMENT_INFO_TOKEN,
        '_createCustomerOrder' => $self->_buildCustomerOrder(),
    };

    my $ThrowsError = 0;

    return $self->_createUIBasketOrder( $hReturnValues, $ThrowsError );
}

sub _buildUIBasketOrder_WithError {
    my $self = shift;

    my $hReturnValues = {
        '_makePurchase' => {
            'successful'    => '0',
            'transactionNo' => '123434',
        },
        '_LogPayment'          => undef,
        '_GetPaymentInfoToken' => PAYMENT_INFO_TOKEN,
        '_createCustomerOrder' => $self->_buildCustomerOrder(),
    };

    my $ThrowsError = 1;

    return $self->_createUIBasketOrder( $hReturnValues, $ThrowsError );
}

sub _buildCustomerOrder {
    my $self = shift;

    my $hParams = {
        'ID'          => '19999',
        'PaidOn'      => undef,
        'CancelledOn' => undef,
    };

    return $self->_createCustomerOrder($hParams);
}

sub _createUIBasketOrder {
    my $self = shift;
    my ( $hReturnValues, $ThrowsError ) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Spreedly::UI::BasketOrder');

    my $aLogPaymentParams = [ 'any', 'string', 'any' ];

    $Mockify->addMockWithReturnValueAndParameterCheck( '_LogPayment', $hReturnValues->{'_LogPayment'}, $aLogPaymentParams );

    $Mockify->addMock(
        '_makePurchase',
        sub {
            shift;
            if ($ThrowsError) {
                Error('ConnectionError');
            }

            return $hReturnValues->{'_makePurchase'};
        }
    );
    my $aRetrieveLineItemPaymentParams = [ { 'object' => 'DE_EPAGES::Presentation::Servlet::BasePermission' } ];
    my $aRetrieveLineItemPaymentReturn = [ { 'object' => 'DE_EPAGES::Spreedly::API::Object::LineItemPaymentSpreedly' } ];
    $Mockify->addMockWithReturnValueAndParameterCheck( '_RetrieveLineItemPayment', $aRetrieveLineItemPaymentReturn, $aRetrieveLineItemPaymentParams );

    $Mockify->addMockWithReturnValueAndParameterCheck( '_GetPaymentInfoToken', $hReturnValues->{'_GetPaymentInfoToken'}, ['object'] );
    $aLogPaymentParams = [ 'object', 'object' ];
    $Mockify->addMockWithReturnValueAndParameterCheck( '_createCustomerOrder', $hReturnValues->{'_createCustomerOrder'}, $aLogPaymentParams );

    my $aPaymentInformationControllerParams = [ 'any', 'string' ];
    $Mockify->addMockWithReturnValueAndParameterCheck( '_StoreTokenFromController', undef, $aPaymentInformationControllerParams );

    my $aShowCustomerOrderParams = [ 'object', 'object' ];
    $Mockify->addMockWithReturnValueAndParameterCheck( '_ShowCustomerOrder', undef, $aShowCustomerOrderParams );

    my $aShowPaymentFailureParams = [ 'object', 'object' ];
    $Mockify->addMockWithReturnValueAndParameterCheck( '_ShowPaymentFailure', undef, $aShowPaymentFailureParams );

    $Mockify->addMethodSpyWithParameterCheck( 'RetrievePaymentInfoToken', ['object'] );
    $Mockify->addMethodSpyWithParameterCheck( '_GetOrder',                ['object'] );
    $Mockify->addMethodSpyWithParameterCheck( '_SaveTransactionNumber',   [ 'any', 'object' ] );
    $Mockify->addMethodSpyWithParameterCheck( '_SetPaymentAsFailed',      [ 'object' ] );

    return $Mockify->getMockObject();
}

sub _createBasket {
    my $self        = shift;
    my $hAttributes = {
        'Site'              => undef,
        'LineItemContainer' => $self->_createLineItemContainer(),
        'GUID'              => '56D5521B-08D3-2F44-728A-7F0001012E94',
    };

    my $Basket = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::Basket', $hAttributes );

    return $Basket;
}

sub _createCustomerOrder {
    my $self = shift;
    my ($hParams) = @_;

    my $hAtributes = {
        'PaidOn'            => $hParams->{'PaidOn'},
        'CancelledOn'       => $hParams->{'CancelledOn'},
        'LineItemContainer' => $self->_createLineItemContainer(),
    };
    my $CustomerOrder = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::CustomerOrder', $hAtributes );

    return $CustomerOrder;
}

sub _createForm {
    my $self = shift;

    my $MockedForm = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::WebInterface::API::Form');

    $MockedForm->mock(
        'executeFormError',
        sub {
            shift;
            my ($hFormError) = @_;

            my $ErrorReason = $hFormError->{'Reason'};
            Error($ErrorReason);

            return;
        }
    );

    $MockedForm->mock(
        'value',
        sub {
            shift;

            return 'ViewMultiCheckoutPayment';
        }
    );

    return $MockedForm;
}

sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes = {
        'GrandTotal' => '80',
        'CurrencyID' => 'GBP',
        'Payment'    => $self->_createLineItemPayment(),
    };

    my $MockedLineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes );

    return $MockedLineItemContainer;
}

sub _createLineItemPayment {
    my $self = shift;

    my $hAttributes = { 'TransTime' => 'Feb 23, 2015 11:44:41 AM', };
    my $MockedLineItemPayment = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Spreedly::API::Object::LineItemPaymentSpreedly', $hAttributes );
    $MockedLineItemPayment->mock(
        'updateTransactionStatus',
        sub {
            shift;

            return;
        }
    );

    return $MockedLineItemPayment;
}

sub _createServlet {
    my $self = shift;
    my ($PaymentInfoToken) = @_;

    my $Basket         = $self->_createBasket();
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Basket);
    $MockifyServlet->addMockWithReturnValue( 'form', $self->_createForm() );
    $MockifyServlet->addParam( 'payment_method_token', $PaymentInfoToken );

    return $MockifyServlet->getMockObject();
}

__PACKAGE__->runScript();

1;
